@@ -7,6 +7,7 @@
 
 #include <cairo.h>
 #include <gtk/gtk.h>
+#include <hildon/hildon.h>
 #include <microfeed-common/microfeedconfiguration.h>
 #include <microfeed-subscriber/microfeedsubscriber.h>
 #include <microfeed-common/microfeedmisc.h>
@@ -18,7 +19,7 @@
 #include "mauku-widget.h"
 
 #define CYCLES 30
-#define PUBLISHER_ID "testtwitter:org.microfeed.Provider.Twitter"
+#define SUBSCRIBER_ID "com.innologies.mauku_widget"
 #define HORIZONTAL_PADDING 8
 #define VERTICAL_PADDING 16
 
@@ -182,12 +183,16 @@ static gboolean view_show_next_timeout(void* user_data) {
 	if (!b->shown) {	
 		mauku_widget->priv->current_badge = b;
 		view_hide_show(mauku_widget);
-		if (!mauku_widget->priv->timeout) {
-			mauku_widget->priv->timeout = g_timeout_add(5000, view_show_next_timeout, mauku_widget);
-		} else if (mauku_widget->priv->current_badge->previous) {
+		if (mauku_widget->priv->current_badge->previous) {
 			more_unshown = TRUE;
+			if (!mauku_widget->priv->timeout) {
+				mauku_widget->priv->timeout = g_timeout_add(5000, view_show_next_timeout, mauku_widget);
+			}
 		}
 	}
+	if (!more_unshown) {
+		mauku_widget->priv->timeout = 0;
+	}
 
 	return more_unshown;
 }
@@ -216,7 +221,6 @@ static void feed_subscribed(MicrofeedSubscriber* subscriber, const char* publish
 		printf("Subscribed: %s %s\n", publisher, uri);
 	
 		microfeed_subscriber_republish_items(subscriber, publisher, uri, NULL, NULL, 1, NULL, NULL);
-		microfeed_subscriber_update_feed(subscriber, publisher, uri, NULL, NULL);
 	}
 }
 
@@ -250,15 +254,15 @@ static void item_added(MicrofeedSubscriber* subscriber, const char* publisher, c
 		}
 		if (!(thread_uri = microfeed_item_get_property(item, "thread.uri")) &&
 		    !(thread_uri = microfeed_item_get_property(item, "feed.uri")) &&
-		    !(thread_uri = microfeed_item_get_property(item, "sender.uri"))) {
+		    !(thread_uri = microfeed_item_get_property(item, "user.uri"))) {
 			thread_uri = uri;
 		}
-		if ((sender_image = microfeed_item_get_property(item, "sender.image")) &&
+		if ((sender_image = microfeed_item_get_property(item, "user.image")) &&
 		    !image_cache_get_image(mauku_widget, sender_image)) {
 			microfeed_subscriber_send_item_data(subscriber, publisher, MICROFEED_FEED_URI_IMAGES, sender_image, NULL, NULL);
 		}
-		if (!(sender_nick = microfeed_item_get_property(item, "sender.nick"))) {
-			sender_nick = "<anonymous>";
+		if (!(sender_nick = microfeed_item_get_property(item, "user.nick"))) {
+			sender_nick = "<unknown>";
 		}
 		if ((s = strchr(publisher, ':'))) {
 			publisher_part = strndup(publisher, s - publisher);
@@ -330,7 +334,6 @@ static MicrofeedSubscriberCallbacks images_callbacks = {
 	NULL,
 	NULL,
 	NULL,
-	NULL,
 	item_data
 };
 
@@ -340,7 +343,6 @@ static MicrofeedSubscriberCallbacks overview_callbacks = {
 	NULL,
 	NULL,
 	NULL,
-	NULL,
 	item_added,
 	item_changed,
 	item_added, /* item_republished */
@@ -349,7 +351,6 @@ static MicrofeedSubscriberCallbacks overview_callbacks = {
 	NULL
 };
 
-
 gboolean on_configure_event(GtkWidget* widget, GdkEventConfigure* event, gpointer user_data) {
 	MaukuWidget* mauku_widget;
 	static int width = 0;
@@ -595,12 +596,65 @@ gboolean on_button_release_event(GtkWidget* widget, GdkEventButton* event, gpoin
 	return FALSE;
 }
 
-static void on_show_settings(HDHomePluginItem* hdhomepluginitem, gpointer user_data) {
-	GtkWidget* dialog;
+static int is_string_in_array(const char** array, const char* string) {
+	int retvalue = 0;
+	while (*array) {
+		if (!strcmp(*array, string)) {
+			retvalue = 1;
+			break;
+		}
+		array++;
+	}
 	
-	dialog = gtk_dialog_new_with_buttons("Mauku Widget Settings", NULL, GTK_DIALOG_MODAL, GTK_STOCK_OK, NULL);
-	gtk_dialog_run(GTK_DIALOG(dialog));
-	gtk_widget_destroy(dialog);
+	return retvalue;
+}
+
+static void configured_subscribe(MicrofeedSubscriber* subscriber, const char* publisher, void* user_data) {
+	microfeed_subscriber_subscribe_feed(subscriber, publisher, MICROFEED_FEED_URI_IMAGES, &images_callbacks, NULL, user_data);
+	microfeed_subscriber_subscribe_feed(subscriber, publisher, MICROFEED_FEED_URI_OVERVIEW, &overview_callbacks, feed_subscribed, user_data);
+}
+	
+static void configured_unsubscribe(MicrofeedSubscriber* subscriber, const char* publisher, const char* uri, void* user_data) {
+	microfeed_subscriber_unsubscribe_feed(subscriber, publisher, uri, NULL, NULL);
+}
+
+static void on_child_exit(GPid pid, gint status, gpointer user_data) {
+	MaukuWidget* mauku_widget;
+	
+	mauku_widget = MAUKU_WIDGET(user_data);
+	microfeed_subscriber_handle_configured_subscriptions(mauku_widget->priv->subscriber, configured_subscribe, configured_unsubscribe, mauku_widget);
+
+	g_spawn_close_pid(pid);
+}
+
+static void on_show_settings(HDHomePluginItem* hd_home_plugin_item, gpointer user_data) {
+	MaukuWidget* mauku_widget;
+	GtkWidget* about_dialog;
+	GtkWidget* label;
+	const char** subscriptions;
+	gchar* args[3];
+	GPid pid;
+	GtkWidget* message_dialog;
+	
+	mauku_widget = MAUKU_WIDGET(hd_home_plugin_item);
+	about_dialog = hildon_dialog_new_with_buttons("Mauku Widget", NULL, GTK_DIALOG_MODAL, "Configure...", GTK_RESPONSE_ACCEPT, NULL);
+	label = gtk_label_new("");
+	gtk_label_set_markup(GTK_LABEL(label), "<big>Mauku Widget</big>\nCopyright 2009 Henrik Hedberg\n&lt;henrik.hedberg@innologies.fi&gt;");
+	gtk_container_add(GTK_CONTAINER(GTK_DIALOG(about_dialog)->vbox), label);
+	gtk_widget_show_all(about_dialog);
+	if (gtk_dialog_run(GTK_DIALOG(about_dialog)) == GTK_RESPONSE_ACCEPT) {
+		args[0] = "microfeed-configuration";
+		args[1] = SUBSCRIBER_ID;
+		args[2] = NULL;
+		if (g_spawn_async(NULL, args, NULL, G_SPAWN_DO_NOT_REAP_CHILD | G_SPAWN_SEARCH_PATH, NULL, NULL, &pid, NULL)) {
+			g_child_watch_add(pid, on_child_exit, mauku_widget);
+		} else {
+			message_dialog = gtk_message_dialog_new(NULL, GTK_DIALOG_MODAL, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, "Could not launch the microfeed-configure application. Maybe the package is missing?");
+			gtk_dialog_run(GTK_DIALOG(message_dialog));
+			gtk_widget_destroy(message_dialog);
+		}
+	}
+	gtk_widget_destroy(about_dialog);
 }
 
 static void mauku_widget_init(MaukuWidget* mauku_widget) {
@@ -608,6 +662,7 @@ static void mauku_widget_init(MaukuWidget* mauku_widget) {
 	DBusError error;
 	DBusConnection* connection;
 	MicrofeedConfiguration* configuration;
+	const char** subscriptions;
 
 	mauku_widget->priv = G_TYPE_INSTANCE_GET_PRIVATE(mauku_widget, MAUKU_TYPE_WIDGET, MaukuWidgetPrivate);
 
@@ -619,7 +674,7 @@ static void mauku_widget_init(MaukuWidget* mauku_widget) {
 	}
 	gtk_widget_set_double_buffered(GTK_WIDGET(mauku_widget), FALSE);
 
-	mauku_widget->priv->background = cairo_image_surface_create_from_png("/home/hhedberg/.mauku-widget/background.png");
+	mauku_widget->priv->background = cairo_image_surface_create_from_png("/home/user/.mauku-widget/background.png");
 	if (cairo_surface_status(mauku_widget->priv->background) != CAIRO_STATUS_SUCCESS) {
 		cairo_surface_destroy(mauku_widget->priv->background);
 		mauku_widget->priv->background = cairo_image_surface_create_from_png("/usr/share/images/mauku-widget-background.png");
@@ -653,10 +708,8 @@ static void mauku_widget_init(MaukuWidget* mauku_widget) {
 	connection = dbus_bus_get(DBUS_BUS_SESSION, &error);
 	dbus_connection_setup_with_g_main(connection, NULL);
 	configuration = microfeed_configuration_new();
-	mauku_widget->priv->subscriber = microfeed_subscriber_new("/com/innologies/mauku_widget", configuration, connection);
-
-	microfeed_subscriber_subscribe_feed(mauku_widget->priv->subscriber, PUBLISHER_ID, MICROFEED_FEED_URI_IMAGES, &images_callbacks, NULL, mauku_widget);
-	microfeed_subscriber_subscribe_feed(mauku_widget->priv->subscriber, PUBLISHER_ID, MICROFEED_FEED_URI_OVERVIEW, &overview_callbacks, feed_subscribed, mauku_widget);
+	mauku_widget->priv->subscriber = microfeed_subscriber_new(SUBSCRIBER_ID, "/com/innologies/mauku_widget", configuration, connection);
+	microfeed_subscriber_handle_configured_subscriptions(mauku_widget->priv->subscriber, configured_subscribe, configured_unsubscribe, mauku_widget);
 }
 
 static void mauku_widget_class_init(MaukuWidgetClass* klass) {
